@@ -8,6 +8,7 @@ except ImportError:
 import mock
 import os
 import sys
+import tempfile
 import time
 import unittest
 
@@ -24,6 +25,12 @@ class TestFstab(unittest.TestCase):
 
     def setUp(self):
         self.testdir = os.path.dirname(os.path.abspath(__file__))
+        self.originalMount = AptBtrfsSnapshot.MOUNTPOINT
+        AptBtrfsSnapshot.MOUNTPOINT = tempfile.mkdtemp(
+                prefix="apt-btrfs-snapshot-mp-")
+
+    def tearDown(self):
+        AptBtrfsSnapshot.MOUNTPOINT = self.originalMount
 
     @mock.patch('apt_btrfs_snapshot.find_executable')
     def test_fstab_detect_snapshot(self, mock_commands):
@@ -69,14 +76,44 @@ class TestFstab(unittest.TestCase):
                          False)
 
     @mock.patch('apt_btrfs_snapshot.LowLevelCommands')
-    def test_mount_btrfs_root_volume(self, mock_commands):
+    @mock.patch('os.path.ismount')
+    def test_mount_btrfs_root_volume(self, mock_ismount, mock_commands):
+        self.assertIs(mock_ismount, os.path.ismount)
         apt_btrfs = AptBtrfsSnapshot(
             fstab=os.path.join(self.testdir, "data", "fstab"))
         mock_commands.mount.return_value = True
         mock_commands.umount.return_value = True
-        mp = apt_btrfs.mount_btrfs_root_volume()
-        self.assertTrue("apt-btrfs-snapshot-mp-" in mp)
-        self.assertTrue(apt_btrfs.umount_btrfs_root_volume())
+
+        # Already mounted, reuse mount, but unmount it
+        mock_ismount.return_value = True
+        with apt_btrfs as mp:
+            self.assertTrue("apt-btrfs-snapshot-mp-" in mp)
+            self.assertEqual(apt_btrfs.mounted, 1)
+            self.assertTrue(os.path.exists(mp))
+        self.assertFalse(os.path.exists(mp))
+        self.assertEqual(apt_btrfs.mounted, 0)
+        self.assertEqual(apt_btrfs.commands.mount.call_count, 0)
+        self.assertEqual(apt_btrfs.commands.umount.call_count, 1)
+
+        # Not already mounted, mount it
+        mock_ismount.return_value = False
+        with apt_btrfs as mp:
+            self.assertTrue("apt-btrfs-snapshot-mp-" in mp)
+            self.assertEqual(apt_btrfs.mounted, 1)
+            self.assertTrue(os.path.exists(mp))
+        self.assertFalse(os.path.exists(mp))
+        self.assertEqual(apt_btrfs.commands.mount.call_count, 1)
+        self.assertEqual(apt_btrfs.commands.umount.call_count, 2)
+
+        # Nested does not double mount
+        with apt_btrfs as mp:
+            with apt_btrfs as mp2:
+                self.assertEqual(apt_btrfs.mounted, 2)
+                self.assertEqual(mp, mp2)
+            self.assertEqual(apt_btrfs.mounted, 1)
+        self.assertEqual(apt_btrfs.mounted, 0)
+        self.assertEqual(apt_btrfs.commands.mount.call_count, 2)
+        self.assertEqual(apt_btrfs.commands.umount.call_count, 3)
         self.assertFalse(os.path.exists(mp))
 
     @mock.patch('apt_btrfs_snapshot.LowLevelCommands')
@@ -98,10 +135,14 @@ class TestFstab(unittest.TestCase):
         self.assertTrue(len(args), 2)
         self.assertTrue(args[0].endswith("@"))
         self.assertTrue("@apt-snapshot-" in args[1])
+        self.assertEqual(apt_btrfs.commands.mount.call_count, 1)
+        self.assertEqual(apt_btrfs.commands.umount.call_count, 1)
         # again with a additional prefix for the snapshot
         res = apt_btrfs.create_btrfs_root_snapshot("release-upgrade-natty-")
         (args, kwargs) = apt_btrfs.commands.btrfs_subvolume_snapshot.call_args
         self.assertTrue("@apt-snapshot-release-upgrade-natty-" in args[1])
+        self.assertEqual(apt_btrfs.commands.mount.call_count, 2)
+        self.assertEqual(apt_btrfs.commands.umount.call_count, 2)
 
     @mock.patch('apt_btrfs_snapshot.LowLevelCommands')
     def test_btrfs_delete_snapshot(self, mock_commands):
@@ -119,6 +160,8 @@ class TestFstab(unittest.TestCase):
         self.assertTrue(apt_btrfs.commands.btrfs_delete_snapshot.called)
         (args, kwargs) = apt_btrfs.commands.btrfs_delete_snapshot.call_args
         self.assertTrue(args[0].endswith("/lala"))
+        self.assertEqual(apt_btrfs.commands.mount.call_count, 1)
+        self.assertEqual(apt_btrfs.commands.umount.call_count, 1)
 
     def test_parser_older_than_to_unixtime(self):
         apt_btrfs = AptBtrfsSnapshot(
